@@ -3,7 +3,7 @@
  * The main Horde_Ldap class.
  *
  * Copyright 2003-2007 Tarjej Huse, Jan Wagner, Del Elson, Benedikt Hallinger
- * Copyright 2009-2013 Horde LLC (http://www.horde.org/)
+ * Copyright 2009-2014 Horde LLC (http://www.horde.org/)
  *
  * @package   Ldap
  * @author    Tarjej Huse <tarjei@bergfald.no>
@@ -32,7 +32,7 @@ class Horde_Ldap
      * - filter:         default search filter.
      * - scope:          default search scope.
      * - user:           configuration parameters for {@link findUserDN()},
-     *                   must contain 'uid', and 'filter' or 'objectclass'
+     *                   must contain 'uid', and may contain 'basedn'
      *                   entries.
      * - auto_reconnect: if true, the class will automatically
      *                   attempt to reconnect to the LDAP server in certain
@@ -206,10 +206,10 @@ class Horde_Ldap
     public function bind($dn = null, $password = null)
     {
         /* Fetch current bind credentials. */
-        if (empty($dn)) {
+        if (is_null($dn)) {
             $dn = $this->_config['binddn'];
         }
-        if (empty($password)) {
+        if (is_null($password)) {
             $password = $this->_config['bindpw'];
         }
 
@@ -877,7 +877,9 @@ class Horde_Ldap
             array(Horde_Ldap_Filter::build($this->_config['user']),
                   Horde_Ldap_Filter::create($this->_config['user']['uid'], 'equals', $user)));
         $search = $this->search(
-            null,
+            isset($this->_config['user']['basedn'])
+                ? $this->_config['user']['basedn']
+                : null,
             $filter,
             array('attributes' => array($this->_config['user']['uid'])));
         if (!$search->count()) {
@@ -1486,7 +1488,7 @@ class Horde_Ldap
                 return sprintf('(|(%s=%s*)(%s=* %s*))', $lhs, self::quote($rhs), $lhs, self::quote($rhs));
             }
             if (!empty($params['approximate'])) {
-                return sprintf('(%s=~%s)', $lhs, self::quote($rhs));
+                return sprintf('(%s~=%s)', $lhs, self::quote($rhs));
             }
             return sprintf('(%s=*%s*)', $lhs, self::quote($rhs));
 
@@ -1518,31 +1520,77 @@ class Horde_Ldap
      *                      name and that attribute's value which make
      *                      up the DN. Example:
      *                      <code>
-     *                      $parts = array(0 => array('cn', 'John Smith'),
-     *                                     1 => array('dc', 'example'),
-     *                                     2 => array('dc', 'com'));
+     *                      $parts = array(
+     *                          array('cn', 'John Smith'),
+     *                          array('dc', 'example'),
+     *                          array('dc', 'com')
+     *                      );
      *                      </code>
+     *                      Nested arrays are supported since 2.1.0, to form
+     *                      multi-valued RDNs. Example:
+     *                      <code>
+     *                      $parts = array(
+     *                          array(
+     *                              array('cn', 'John'),
+     *                              array('sn', 'Smith'),
+     *                              array('o', 'Acme Inc.'),
+     *                          ),
+     *                          array('dc', 'example'),
+     *                          array('dc', 'com')
+     *                      );
+     *                      </code>
+     *                      which will result in
+     *                      cn=John+sn=Smith+o=Acme Inc.,dc=example,dc=com
      *
      * @return string  The properly quoted string DN.
      */
     public static function quoteDN($parts)
     {
-        $dn = '';
-        $count = count($parts);
-        for ($i = 0; $i < $count; $i++) {
-            if ($i > 0) {
-                $dn .= ',';
-            }
-            $dn .= $parts[$i][0] . '=';
+        return implode(',', array_map('self::_quoteRDNs', $parts));
+    }
 
-            // See if we need to quote the value.
-            if (preg_match('/^\s|\s$|\s\s|[,+="\r\n<>#;]/', $parts[$i][1])) {
-                $dn .= '"' . str_replace('"', '\\"', $parts[$i][1]) . '"';
-            } else {
-                $dn .= $parts[$i][1];
-            }
+    /**
+     * Takes a single or a list of RDN arrays with an attribute name and value
+     * and properly quotes it according to RFC 1485.
+     *
+     * @param array $attribute  A tuple or array of tuples containing the
+     *                          attribute name and that attribute's value which
+     *                          make up the RDN.
+     *
+     * @return string  The properly quoted string RDN.
+     */
+    protected static function _quoteRDNs($attribute)
+    {
+        if (is_array($attribute[0])) {
+            return implode(
+                '+',
+                array_map('self::_quoteRDN', $attribute)
+            );
+        } else {
+            return self::_quoteRDN($attribute);
+        }
+    }
+
+    /**
+     * Takes an RDN array with an attribute name and value and properly quotes
+     * it according to RFC 1485.
+     *
+     * @param array $attribute  A tuple containing the attribute name and that
+     *                          attribute's value which make up the RDN.
+     *
+     * @return string  The properly quoted string RDN.
+     */
+    protected static function _quoteRDN($attribute)
+    {
+        $rdn = $attribute[0] . '=';
+
+        // See if we need to quote the value.
+        if (preg_match('/^\s|\s$|\s\s|[,+="\r\n<>#;]/', $attribute[1])) {
+            $rdn .= '"' . str_replace('"', '\\"', $attribute[1]) . '"';
+        } else {
+            $rdn .= $attribute[1];
         }
 
-        return $dn;
+        return $rdn;
     }
 }
