@@ -3,7 +3,7 @@
 require_once __DIR__ . '/TestBase.php';
 
 /**
- * Copyright 2010-2013 Horde LLC (http://www.horde.org/)
+ * Copyright 2010-2014 Horde LLC (http://www.horde.org/)
  *
  * @package    Ldap
  * @subpackage UnitTests
@@ -83,6 +83,19 @@ class Horde_Ldap_LdapTest extends Horde_Ldap_TestBase
         $ldap = new Horde_Ldap($lcfg);
     }
 
+    /**
+     * Tests if the server can connect and bind, but not rebind with empty
+     * password.
+     *
+     * @expectedException Horde_Ldap_Exception
+     */
+    public function testConnectAndEmptyRebind()
+    {
+        // Simple working connect and privileged bind.
+        $ldap = new Horde_Ldap(self::$ldapcfg['server']);
+        $ldap->bind(self::$ldapcfg['server']['binddn'], '');
+    }
+
     /**
      * Tests startTLS() if server supports it.
      */
@@ -608,4 +621,44 @@ class Horde_Ldap_LdapTest extends Horde_Ldap_TestBase
         $ldap = new Horde_Ldap(self::$ldapcfg['server']);
         $this->assertTrue(is_resource($ldap->getLink()));
     }
+
+    public function testQuoteDN()
+    {
+        $this->assertEquals(
+            'cn=John Smith,dc=example,dc=com',
+            Horde_Ldap::quoteDN(
+                array(
+                    array('cn', 'John Smith'),
+                    array('dc', 'example'),
+                    array('dc', 'com')
+                )
+            )
+        );
+        $this->assertEquals(
+            'cn=John+sn=Smith+o=Acme Inc.,dc=example,dc=com',
+            Horde_Ldap::quoteDN(
+                array(
+                    array(
+                        array('cn', 'John'),
+                        array('sn', 'Smith'),
+                        array('o', 'Acme Inc.'),
+                    ),
+                    array('dc', 'example'),
+                    array('dc', 'com')
+                )
+            )
+        );
+        $this->assertEquals(
+            'cn=John+sn=Smith+o=Acme Inc.',
+            Horde_Ldap::quoteDN(
+                array(
+                    array(
+                        array('cn', 'John'),
+                        array('sn', 'Smith'),
+                        array('o', 'Acme Inc.'),
+                    ),
+                )
+            )
+        );
+    }
 }
